﻿using ImageMagick;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Gifitti.Models
{
    class GifModel
    {
        public Image gifImage { get; set; }
        public Image originalGif { get; private set; }
        private FrameDimension dimension;
        public int numberOfFrames { get; private set; }
        private Image[] frames;
        private int step = 1;
        private int currentFrame = -1;
        private bool reverse;
        public int startFrame = 0;
        public int endFrame = 0;

        //constructor
        public GifModel(String path)
        {
            //Save the original Gif
            originalGif = Image.FromFile(path);
            this.gifImage = originalGif.Clone() as Image;
            dimension = new FrameDimension(originalGif.FrameDimensionsList[0]);
            frameConstruction(originalGif);
        }

        private void frameConstruction(Image originalImg)
        {
            numberOfFrames = originalImg.GetFrameCount(FrameDimension.Time);
            endFrame = numberOfFrames;
            frames = new Image[numberOfFrames];

            for(int i = 0; i < numberOfFrames; i++)
            {
                originalGif.SelectActiveFrame(FrameDimension.Time, i);
                frames[i] = ((Image)originalGif.Clone());
            }
        }

        /// <summary>
        /// Changes the loaded GIF to be what was originally loaded by the User.
        /// If a new file is opened the current original GIF is overloaded.
        /// </summary>
        public void resetToOriginalGif()
        {
            gifImage = originalGif.Clone() as Image;
        }

        /// <summary>
        /// Takes the full path of a GIF save location, 
        /// extension included
        /// </summary>
        /// <param name="path">String representation of path to save the gif to. 
        ///     Name and extension of gif mandatory.
        /// </param>
        /// <exception cref="DirectoryNotFoundException">
        /// Thrown if the path is not valid for the local system,
        /// OR the file has an extension other than [G|g][I|i][F|f]
        /// </exception>
        public void saveGif(string path)
        {
            //Error Handel on path
            Regex savePathTest = new Regex(@"^(([^\/\\]+)[\/\\])*([^\/\\]*)\.[g|G][i|I][f|F]$");
            if (!savePathTest.IsMatch(path))
            {
                throw new DirectoryNotFoundException();
            }

            //TODO take in image setting components as well as frames being viewed etc.
            //Build GIF
            using (MagickImageCollection collection = new MagickImageCollection())
            {
                for (int i = 0; i < frames.Length; i++)
                    collection.Add(new MagickImage(frames[i] as Bitmap));
                
                // Maximize Colour saturation
                QuantizeSettings settings = new QuantizeSettings();
                settings.Colors = 256;
                collection.Quantize(settings);

                // Optimized the GIF being exported
                collection.Optimize();
                // Save GIF to path
                collection.Write(path);
            }
        }

        public void resizeGif(int newWidth, int newHeight)
        {
            using (MagickImageCollection collection = new MagickImageCollection())
            {
                for (int i = 0; i < frames.Length; i++)
                {
                    collection.Add(new MagickImage(frames[i] as Bitmap));
                    collection[i].Resize(newWidth, newHeight);
                }
                // Optionally reduce colors
                QuantizeSettings settings = new QuantizeSettings();
                settings.Colors = 256;
                collection.Quantize(settings);

                // Optionally optimize the images (images should have the same size).
                collection.Optimize();

                // Save gif
                collection.Write("tempResize.gif");
            }
        }

        public void returnSubGif(int start, int stop)
        {
            using (MagickImageCollection collection = new MagickImageCollection())
            {
                // Add first image and set the animation delay to 100ms
                for (int i = start; i < stop; i++)
                {
                    collection.Add(new ImageMagick.MagickImage(frames[i] as Bitmap));
                    collection[i - start].AnimationDelay = 1;
                }
                // Optionally reduce colors
                QuantizeSettings settings = new QuantizeSettings();
                settings.Colors = 256;
                collection.Quantize(settings);

                // Optionally optimize the images (images should have the same size).
                collection.Optimize();

                // Save gif
                collection.Write("temp.gif");
            }
        }

        public bool ReverseAtEnd
        {
            //whether the gif should play backwards when it reaches the end
            get { return reverse; }
            set { reverse = value; }
        }

        public Image GetNextFrame()
        {

            currentFrame += step;

            //if the animation reaches a boundary...
            if (currentFrame >= numberOfFrames || currentFrame < 1 || currentFrame >= endFrame)
            {
                if (reverse)
                {
                    step *= -1;
                    //...reverse the count
                    //apply it
                    currentFrame += step;
                }
                else
                {
                    currentFrame = startFrame;
                    //...or start over
                }
            }
            return GetFrame(currentFrame);
        }

        public Image GetFrame(int index)
        {
            originalGif.SelectActiveFrame(dimension, index);
            //find the frame
            return (Image)originalGif.Clone();
            //return a copy of it
        }
    }
}
